# Edit this configuration file to define what should be installed on
# your system.  Help is available in the configuration.nix(5) man page
# and in the NixOS manual (accessible by running ‘nixos-help’).

{ config, pkgs, ... }:
let
  secrets = (import ./secrets.nix).settings;
in
{
  imports =
    [ # Include the results of the hardware scan.
      ./hardware-configuration.nix
      ./network.nix
      ./packages.nix
      ./services.nix
      ./syncthing.nix
      ./users.nix
    ];

  # Use the GRUB 2 boot loader.
  boot = {
    loader = {
      grub = {
        enable = true;
        version = 2;
        device = "/dev/sda";
      };
      timeout = 10;
    };
    kernel.sysctl = {
      "net.ipv4.forwarding" = true;
      "fs.inotify.max_user_watches" = 524288;
      "fs.file-max" = 524288;
      "vm.nr_hugepages" = 3072;
    };
    kernelParams = [ "radeon.si_support=0" "amdgpu.si_support=1" "radeon.cik_support=0" "amdgpu.cik_support=1" ];
    kernelPackages = pkgs.linuxPackages_latest;
  };

  hardware = {
    enableRedistributableFirmware = true;
    # enableAllFirmware = true;
    opengl = {
      enable = true;
      driSupport = true;
      driSupport32Bit = true;
      setLdLibraryPath = true;
    };
    opengl.extraPackages = with pkgs; [
      libvdpau-va-gl
      libva
      ocl-icd
      libclc
      mesa.opencl
    ];
  };

  powerManagement.cpuFreqGovernor = "performance";

  system = {
    stateVersion = "22.05";
    autoUpgrade.enable = true;
    autoUpgrade.allowReboot = true;
  };

  fileSystems."/srv" = {
    device = "/dev/disk/by-label/media";
    fsType = "btrfs";
    options = [ "compress=zstd" ]; 
  };

  fileSystems."/export/cloud" = {
    device = "/srv/media/cloud";
    options = [ "bind" ];
  };

  fileSystems."/export/plex" = {
    device = "/srv/media/plex";
    options = [ "bind" ];
  };

  nixpkgs.config = {
    allowBroken = false;
    allowUnfree = true;
  };
  
  nix = {
    settings = {
      auto-optimise-store = true;
    };
    gc = {
      automatic = true;
      dates = "weekly";
      options = "--delete-older-than 14d";
    };
  };
  
  fonts.enableDefaultFonts = true;

  security = {
    doas = {
      enable = true;
      wheelNeedsPassword = false;
    };
    # wrappers."mount.nfs".source = "${pkgs.nfs-utils.out}/bin/mount.nfs";
    pam.loginLimits = [
      { domain = "*"; item = "nofile"; type = "soft"; value = "524288"; }
      { domain = "*"; item = "nofile"; type = "hard"; value = "524288"; }
    ];
  };

  environment.variables = rec {
           LIBVA_DRIVERS_PATH                 = "/run/opengl-driver/lib/dri";
           CLOVER_PLATFORM_VERSION_OVERRIDE   = "1.2";
           CLOVER_DEVICE_VERSION_OVERRIDE     = "1.2";
           CLOVER_DEVICE_CLC_VERSION_OVERRIDE = "1.2";
  };

  programs = {
    mtr.enable = true;
    gnupg.agent = {
      enable = true;
      enableSSHSupport = true;
    };
    bash.enableCompletion = true;
    fish.enable = true;
  };

  time.timeZone = "Europe/Moscow";
}

