{ config, pkgs, ... }:
let
  secrets = (import ./secrets.nix).settings;
in  
{
  services = {
    # btc on i2p require v22.0+
    # monero explorer
    # bwt as nixos package
    # nextcloud
    acpid.enable = true;
    apcupsd = {
      enable = true;
      configText = ''
        UPSTYPE usb
        NISIP 127.0.0.1
        BATTERYLEVEL 7
        BEEPSTATE N
      '';
    };
    #localtime.enable = true;
    locate.enable = true;
    nfs.server.enable = true;
    openssh.enable = true;
    timesyncd.enable = true;
    tlp.enable = true;
    avahi = {
      enable = true;
      nssmdns = true;
    };
    bitcoind."default" = {
      enable = true;
      port = secrets.ports.btc.main;
      rpc.port = secrets.ports.btc.rpc;
      user = "bitcoin";
      group = "bitcoin";
      dataDir = "/srv/bitcoin/data";
      configFile = "/srv/bitcoin/config/bitcoin.conf";
    };
    btrfs.autoScrub = {
      enable = true;
      fileSystems = [ "/srv" ]; 
    };
    deluge = {
      enable = true;
      declarative = true;
      user = secrets.plex.user;
      group = secrets.plex.group;
      config = {
        download_location = "/srv/media/cloud/torrent/Incomplete";
        move_completed_path = "/srv/media/cloud/torrent/Downloads";
        torrentfiles_location = "/srv/media/cloud/torrent";
        allow_remote = true;
        daemon_port = secrets.ports.deluge.daemon;
        listen_ports = secrets.ports.deluge.listen;
        outgoing_ports = secrets.ports.deluge.outgoing;
        upnp = false;
        natpmp = false;
        dht = false;
        proxy = {
          type = 4;
          hostname = secrets.wg_settings.host_ip;
          port = secrets.ports.tinyproxy;
          proxy_hostnames = true;
          proxy_peer_connections = true;
          proxy_tracker_connections = true;
          force_proxy = true;
          anonymous_mode = false;
        };
      };
      openFirewall = true;
      dataDir = "/srv/deluge";
      authFile = "/srv/deluge/auth";
      web = {
        enable = false;
        port = secrets.ports.deluge.web;
        openFirewall = true;
      };
    };
    jellyfin = {
      user = secrets.plex.user;
      group = secrets.plex.group;
      enable = true;
      openFirewall = true;
    };
    plex = {
      user = secrets.plex.user;
      group = secrets.plex.group;
      dataDir = "/srv/plex";
      enable = true;
      openFirewall = true;
    };
    syncthing = {
      enable = true;
      user = "syncthing";
      dataDir = "/srv/syncthing/data";
      configDir = "/srv/syncthing/config";
      guiAddress = "${secrets.ip.address}:${toString secrets.ports.syncthing}";
      relay.listenAddress = "${secrets.wg_settings.ip}";

      openDefaultPorts = true;
      overrideDevices = true;
      overrideFolders = true;
      #todo: use tor as socks proxy
    };
    transmission = {
      enable = true;
      user = secrets.plex.user;
      group = secrets.plex.group;
      home = "/srv/transmission";
      openPeerPorts = true;
      openRPCPort = true;
      downloadDirPermissions = "775";
      settings = {
        rpc-whitelist = secrets.transmission.whitelist;
        watch-dir = "/srv/media/cloud/torrent";
        watch-dir-enabled = true;
        rpc-bind-address = secrets.ip.address;
        rpc-port = secrets.ports.transmission.rpc;
        peer-port-random-low = secrets.ports.transmission.peer-low;
        peer-port-random-high = secrets.ports.transmission.peer-high;
        peer-port-random-on-start = true;
        speed-limit-down-enabled = true;
        speed-limit-up-enabled = true;
        speed-limit-up = 200;
        speed-limit-down = 600;
        incomplete-dir = "/srv/media/cloud/torrent/Incomplete";
        download-dir = "/srv/media/cloud/torrent/Downloads";
      };
    };
    # TODO: specify explicitly hostname
    tor = {
      enable = true;
      openFirewall = true;
      enableGeoIP = false;
      settings = {
        CookieAuthentication = true;
        CookieAuthFileGroupReadable = true;
        DataDirectoryGroupReadable = true;
        CacheDirectoryGroupReadable = true;
        ControlPort = secrets.ports.tor.control;
      };
      client = {
        enable = true;
      };
    };
    monero = {
      enable = false;
      dataDir = "/srv/monero/data";
      rpc = {
        address = secrets.wg_settings.ip;
        restricted = false;
        port = secrets.ports.monero.rpc-private;
        user = secrets.monero.user;
        password = secrets.monero.password;
      };
      limits = {
        upload = 1048576;
        download = 1048576;
        threads = 2;
      };
      extraConfig = ''
        check-updates=disabled
        public-node=1
        confirm-external-bind=1
        rpc-ssl=autodetect
        db-sync-mode=safe
        out-peers=128
        in-peers=128
        # P2P full node
        p2p-bind-ip=0.0.0.0
        p2p-bind-port=${toString secrets.ports.monero.p2p-public}
        tx-proxy=i2p,${secrets.wg_settings.host_ip}:${toString secrets.ports.i2pd.socks},64
        tx-proxy=tor,${secrets.wg_settings.host_ip}:${toString secrets.ports.tor.main},64
        rpc-restricted-bind-ip=0.0.0.0
        rpc-restricted-bind-port=${toString secrets.ports.monero.rpc}
        no-igd=1
        no-zmq=1
        max-txpool-weight=268435456
        # Tor: add P2P seed nodes for the Tor network
        # For an up-to-date list of working nodes see https://www.ditatompel.com/monero/node-peers
        add-peer=4egylyolrzsk6rskorqvocipdo4tqqoyzxnplbjorns7issmgpoxvtyd.onion:18083
        add-peer=fagmobguo6u4z4b2ghyg3jegcdpmd4qj4wxkhemph5d5q6dltllveqyd.onion:18083
        add-peer=monerokdwzyuml7vfp73fjx5277lzesbrq4nvbl3r3t5ctgodsx34vid.onion:18089
        add-peer=b75obarnhi42p7js7wgzo7v3wtiwcgf4bknrwv6ihatop77jivrtwpid.onion:15892
        add-peer=5nvd6jbefgto3u74nzzdkcsbqgxyzrkk7bz5qupsdqg4gbuj5valiaqd.onion:18083
        add-peer=ozeavjybjbxbvmfcpxzjcn4zklbgohjwwndzenjt44pypvx6jisy74id.onion:18083
        add-peer=xcccrsxi2zknef6zl3sviasqg4xnlkh5k3xqu7ytwkpfli3huyfvsjid.onion:18083

        # Make the seed nodes permanent to fix monerod issue of not maintaining enough connections,
        # based on this reddit comment:
        # https://www.reddit.com/r/monerosupport/comments/k3m3x2/comment/ge5ehcy/?utm_source=share&utm_medium=web2x&context=3
        add-priority-node=4egylyolrzsk6rskorqvocipdo4tqqoyzxnplbjorns7issmgpoxvtyd.onion:18083
        add-priority-node=fagmobguo6u4z4b2ghyg3jegcdpmd4qj4wxkhemph5d5q6dltllveqyd.onion:18083
        add-priority-node=monerokdwzyuml7vfp73fjx5277lzesbrq4nvbl3r3t5ctgodsx34vid.onion:18089
        add-priority-node=b75obarnhi42p7js7wgzo7v3wtiwcgf4bknrwv6ihatop77jivrtwpid.onion:15892
        add-priority-node=5nvd6jbefgto3u74nzzdkcsbqgxyzrkk7bz5qupsdqg4gbuj5valiaqd.onion:18083
        add-priority-node=ozeavjybjbxbvmfcpxzjcn4zklbgohjwwndzenjt44pypvx6jisy74id.onion:18083
        add-priority-node=xcccrsxi2zknef6zl3sviasqg4xnlkh5k3xqu7ytwkpfli3huyfvsjid.onion:18083

        add-peer=core5hzivg4v5ttxbor4a3haja6dssksqsmiootlptnsrfsgwqqa.b32.i2p
        add-peer=dsc7fyzzultm7y6pmx2avu6tze3usc7d27nkbzs5qwuujplxcmzq.b32.i2p
        add-peer=spn4ef3gddveqxkco6fhh7epyxnvdmei4w6hioiiunfvvljxgcoa.b32.i2p
        add-peer=sel36x6fibfzujwvt4hf5gxolz6kd3jpvbjqg6o3ud2xtionyl2q.b32.i2p
        add-peer=yht4tm2slhyue42zy5p2dn3sft2ffjjrpuy7oc2lpbhifcidml4q.b32.i2p
        add-peer=4q6ps46l3wv2x6zn7faeliycpdwldohex5oc4slplud65o6lpleq.b32.i2p
        add-priority-node=s3l6ke4ed3df466khuebb4poienoingwof7oxtbo6j4n56sghe3a.b32.i2p

        anonymous-inbound=${secrets.monero.i2p-p2p},${secrets.wg_settings.ip}:${toString secrets.ports.monero.p2p-i2p},64
        anonymous-inbound=${secrets.monero.tor}:${toString secrets.ports.monero.p2p-tor},${secrets.wg_settings.ip}:${toString secrets.ports.monero.p2p-tor},64
        disable-rpc-ban=1
     '';
    };
  };
  systemd.services.tor.serviceConfig.User = "tor";
  systemd.services.tor.serviceConfig.Group = "tor";
}
